/*******************************************************************************
 * Copyright (c) 2010, 2013 Tasktop Technologies
 * Copyright (c) 2010, 2011 SpringSource, a division of VMware
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Tasktop Technologies - initial API and implementation
 ******************************************************************************/
package com.tasktop.c2c.server.deployment.service;

import java.util.List;

import com.tasktop.c2c.server.common.service.EntityNotFoundException;
import com.tasktop.c2c.server.common.service.ValidationException;
import com.tasktop.c2c.server.common.service.domain.Region;
import com.tasktop.c2c.server.common.service.logging.NoLog;
import com.tasktop.c2c.server.deployment.domain.CloudService;
import com.tasktop.c2c.server.deployment.domain.DeploymentConfigActivity;
import com.tasktop.c2c.server.deployment.domain.DeploymentConfiguration;
import com.tasktop.c2c.server.deployment.domain.DeploymentServiceConfiguration;
import com.tasktop.c2c.server.deployment.domain.DeploymentServiceType;
import com.tasktop.c2c.server.profile.domain.build.BuildDetails;

/**
 * Service for interacting with {@link DeploymentConfiguration}s as well as operating on them to perform the actual
 * deployment. Note this service is project-specific.
 * 
 * @author Clint Morgan <clint.morgan@tasktop.com> (Tasktop Technologies Inc.)
 * @author Terry Denney <terry.denney@tasktop.com> (Tasktop Technologies Inc.)
 * 
 */
public interface DeploymentConfigurationService {
	/**
	 * List the current deployment configurations. If there are errors communicating with CF for a particular
	 * {@link DeploymentConfiguration}, then the errorMessage will be populated.
	 * 
	 * @param region
	 * @return
	 */
	List<DeploymentConfiguration> listDeployments(Region region);

	/**
	 * List the current deployment configurations. The underlying deployment service will not be queried, so status
	 * information will not be populated.
	 * 
	 * @param region
	 * @return
	 */
	List<DeploymentConfiguration> listDeploymentsWithoutServiceQuery(Region region);

	/**
	 * Check if credentials are valid.
	 * 
	 * @param url
	 * @param username
	 * @param password
	 * @return true for valid credentials, false for invalid
	 */
	boolean validateCredentials(String url, String username, @NoLog String password) throws ServiceException;

	/**
	 * Create a deployment configuration.
	 * 
	 * 
	 * @param deploymentConfiguration
	 * @return
	 * @throws ValidationException
	 */
	DeploymentConfiguration createDeployment(DeploymentConfiguration deploymentConfiguration,
                boolean mayDeployOnCreate)
			throws ValidationException;

	/**
	 * Fetch a deployment configuration.
	 * 
	 * @param deploymentConfigurationId
	 * @throws EntityNotFoundException
	 * @throws ServiceException
	 */

	DeploymentConfiguration fetchDeployment(Long deploymentConfigurationId)
			throws EntityNotFoundException, ServiceException;

	/**
	 * Update a deployment configuration.
	 * 
	 * @param deploymentConfiguration
	 * @return
	 * @throws EntityNotFoundException
	 * @throws ValidationException
	 */
	DeploymentConfiguration updateDeployment(DeploymentConfiguration deploymentConfiguration,
                boolean mayDeploy)
			throws EntityNotFoundException, ValidationException;

	/**
	 * Delete a deployment configuration.
	 * 
	 * @param deploymentConfiguration
	 * @param alsoDeleteFromCloudFoundry
	 *            if true, then we will propogate the delete to CF.
	 * @throws EntityNotFoundException
	 * @throws ServiceException
	 */
	void deleteDeployment(Long deploymentConfigurationId, boolean alsoDeleteFromCloudFoundry)
			throws EntityNotFoundException, ServiceException;

	/**
	 * Deploy an application
	 * 
	 * @param deploymentConfigurationId
	 * @throws EntityNotFoundException
	 * @throws ServiceException
	 * @throws ValidationException
	 */
	DeploymentConfiguration deploy(Long deploymentConfigurationId) throws EntityNotFoundException, ServiceException,
			ValidationException;

	/**
	 * Deploy an application
	 * 
	 * @param deploymentConfiguration
	 * @throws EntityNotFoundException
	 * @throws ServiceException
	 * @throws ValidationException
	 */
	DeploymentConfiguration deploy(DeploymentConfiguration deploymentConfigurationId) throws EntityNotFoundException, ServiceException,
			ValidationException;

	/**
	 * Undeploy an application
	 * 
	 * @param deploymentConfiguration
	 * @throws EntityNotFoundException
	 * @throws ServiceException
	 * @throws ValidationException
	 */
	DeploymentConfiguration undeploy(Long deploymentConfigurationId) throws EntityNotFoundException, ServiceException,
			ValidationException;

	DeploymentConfiguration startDeployment(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException;

	DeploymentConfiguration stopDeployment(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException;

	DeploymentConfiguration restartDeployment(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException;

	List<CloudService> getAvailableServices(DeploymentConfiguration deploymentConfiguration) throws ServiceException;

	CloudService createService(CloudService service, DeploymentConfiguration deploymentConfiguration)
			throws ServiceException;

	List<Integer> getAvailableMemoryConfigurations(DeploymentConfiguration deploymentCongfiguration)
			throws ServiceException;

	List<DeploymentServiceConfiguration> getAvailableServiceConfigurations(
			DeploymentConfiguration deploymentConfiguration) throws ServiceException;

	/**
	 * To support automatic deployments
	 * 
	 * @param jobName
	 * @param buildDetails
	 */
	void onBuildCompleted(String jobName, BuildDetails buildDetails);

	/**
	 * Get a list of the supported service teyps
	 * 
	 * @return list of supported services
	 */
	List<DeploymentServiceType> getSupportedDeploymentServiceTypes();

	/**
	 * get recent activity on the wiki
	 * 
	 * @return a list of recent activity, or an empty list if there is no activity
	 */
	List<DeploymentConfigActivity> getRecentActivity(Region region);
        

	/**
	 * Process DeploymentConfiguration on entry to the API
	 * 
         * @param deploymentConfiguration
	 * @return a potentially modified DeploymentConfiguration
	 */
	DeploymentConfiguration onEntry(DeploymentConfiguration deploymentConfiguration);  
        
	/**
	 * Process DeploymentConfiguration on entry to the API
	 * 
         * @param deploymentConfiguration
	 * @return a potentially modified DeploymentConfiguration
	 */
	DeploymentConfiguration onExit(DeploymentConfiguration deploymentConfiguration);        

}
