/*******************************************************************************
 * Copyright (c) 2010, 2013 Tasktop Technologies
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     Tasktop Technologies - initial API and implementation
 ******************************************************************************/
package com.tasktop.c2c.server.deployment.service;

import java.util.List;

import com.tasktop.c2c.server.common.service.EntityNotFoundException;
import com.tasktop.c2c.server.common.service.ValidationException;
import com.tasktop.c2c.server.common.service.WrappedCheckedException;
import com.tasktop.c2c.server.common.service.domain.Region;
import com.tasktop.c2c.server.common.service.web.AbstractVersionedRestServiceClient;
import com.tasktop.c2c.server.deployment.domain.CloudService;
import com.tasktop.c2c.server.deployment.domain.DeploymentConfigActivity;
import com.tasktop.c2c.server.deployment.domain.DeploymentConfiguration;
import com.tasktop.c2c.server.deployment.domain.DeploymentServiceConfiguration;
import com.tasktop.c2c.server.deployment.domain.DeploymentServiceType;
import com.tasktop.c2c.server.profile.domain.build.BuildDetails;

/**
 * A client to communicate to the deployment configuration REST webservice.
 * <p/>
 * This client object should be configured with a RestClientDelegate, e.g. ApacheHttpRestClientDelegate, which is
 * provided in SDK, to handle HTTP REST communication.
 * <p/>
 * Usage:
 * 
 * <pre>
 * ProfileWebServiceClient client = new ProfileWebServiceClient();
 * ApacheHttpRestClientDelegate delegate = new ApacheHttpRestClientDelegate(user, passwd);
 * client.setRestClientDelegate(delegate);
 * client.setBaseUrl(REST_API_URL);
 * </pre>
 * 
 * @author rwidyono (Oracle USA Inc.)
 * 
 */
public class DeploymentConfigurationServiceClient extends AbstractVersionedRestServiceClient implements
		DeploymentConfigurationService {

	public static final String PROJECT_IDENTIFIER_URLPARAM = "projectIdentifier";

	public static final String CONFIGURATION_IDENTIFIER_URLPARAM = "configIdentifier";

	protected static class ServiceCallResult {
		List<DeploymentConfiguration> deploymentConfigurationList;
		DeploymentConfiguration deploymentConfiguration;

		/**
		 * @return the deploymentConfigurationList
		 */
		public List<DeploymentConfiguration> getDeploymentConfigurationList() {
			return deploymentConfigurationList;
		}

		/**
		 * @param deploymentConfigurationList
		 *            the deploymentConfigurationList to set
		 */
		public void setDeploymentConfigurationList(List<DeploymentConfiguration> deploymentConfigurationList) {
			this.deploymentConfigurationList = deploymentConfigurationList;
		}

		/**
		 * @return the deploymentConfiguration
		 */
		public DeploymentConfiguration getDeploymentConfiguration() {
			return deploymentConfiguration;
		}

		/**
		 * @param deploymentConfiguration
		 *            the deploymentConfiguration to set
		 */
		public void setDeploymentConfiguration(DeploymentConfiguration deploymentConfiguration) {
			this.deploymentConfiguration = deploymentConfiguration;
		}
	}

	private abstract class GetCall<T> {

		public abstract T getValue(ServiceCallResult result);

		public T doCall(String urlStub, Object... variables) {
			ServiceCallResult callResult = delegate.getForObject(computeUrl(urlStub), ServiceCallResult.class,
					variables);

			T retVal = getValue(callResult);

			if (retVal == null) {
				throw new IllegalStateException("Illegal result from call to deploymentConfigurationService");
			}

			return retVal;
		}
	}

	private abstract class PostCall<T> {

		public abstract T getValue(ServiceCallResult result);

		public T doCall(String urlStub, Object objToPost, Object... variables) {
			ServiceCallResult callResult = delegate.postForObject(computeUrl(urlStub), objToPost,
					ServiceCallResult.class, variables);

			T retVal = getValue(callResult);

			if (retVal == null) {
				throw new IllegalStateException("Illegal result from call to deploymentConfigurationService");
			}

			return retVal;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.common.service.web.VersionedServiceClient#getClientVersion()
	 */
	public String getClientVersion() {
		// TODO Auto-generated method stub
		return null;
	}

	public static final String LIST_DEPLOYMENTS_URL = "list/{" + PROJECT_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#listDeployments(com.tasktop.c2c.server
	 * .common.service.domain.Region)
	 */
	public List<DeploymentConfiguration> listDeployments(Region region) {
		return new GetCall<List<DeploymentConfiguration>>() {
			public List<DeploymentConfiguration> getValue(ServiceCallResult result) {
				return result.getDeploymentConfigurationList();
			}
		}.doCall(LIST_DEPLOYMENTS_URL);
	}

        public static final String LIST_DEPLOYMENT_CONFIGS_URL = "configs/{" + PROJECT_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#listDeploymentsWithoutServiceQuery(com
	 * .tasktop.c2c.server.common.service.domain.Region)
	 */
        public List<DeploymentConfiguration> listDeploymentsWithoutServiceQuery(Region region) {
		return new GetCall<List<DeploymentConfiguration>>() {
			public List<DeploymentConfiguration> getValue(ServiceCallResult result) {
				return result.getDeploymentConfigurationList();
			}
		}.doCall(LIST_DEPLOYMENT_CONFIGS_URL);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#validateCredentials(java.lang.String,
	 * java.lang.String, java.lang.String)
	 */
	public boolean validateCredentials(String url, String username, String password) throws ServiceException {
		// TODO Auto-generated method stub
		return false;
	}

	public static final String CREATE_DEPLOYMENT_URL = "create/{" + PROJECT_IDENTIFIER_URLPARAM + "}";
	public static final String CREATE_DEPLOYMENT_MAY_DEPLOY_URL = "create/{" + PROJECT_IDENTIFIER_URLPARAM + "}/maydeploy";

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#createDeployment(com.tasktop.c2c.server
	 * .deployment.domain.DeploymentConfiguration)
	 */
	public DeploymentConfiguration createDeployment(DeploymentConfiguration deploymentConfiguration, boolean mayDeployOnCreate)
			throws ValidationException {
		try {
                    if (mayDeployOnCreate) {
			return new PostCall<DeploymentConfiguration>() {
				public DeploymentConfiguration getValue(ServiceCallResult result) {
					return result.getDeploymentConfiguration();
				}
			}.doCall(CREATE_DEPLOYMENT_MAY_DEPLOY_URL, deploymentConfiguration);
                    } else {
			return new PostCall<DeploymentConfiguration>() {
				public DeploymentConfiguration getValue(ServiceCallResult result) {
					return result.getDeploymentConfiguration();
				}
			}.doCall(CREATE_DEPLOYMENT_URL, deploymentConfiguration);
                    }
		} catch (WrappedCheckedException e) {
			convertValidationException(e);
			throw (e);
		}
	}

	public static final String FETCH_DEPLOYMENT_URL = "fetch/{" + PROJECT_IDENTIFIER_URLPARAM + "}/config/{"
			+ CONFIGURATION_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#fetchDeployment(java.lang.Long)
	 */
	public DeploymentConfiguration fetchDeployment(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException {
		return new GetCall<DeploymentConfiguration>() {
			public DeploymentConfiguration getValue(ServiceCallResult result) {
				return result.getDeploymentConfiguration();
			}
		}.doCall(FETCH_DEPLOYMENT_URL);
	}

	public static final String UPDATE_DEPLOYMENT_URL = "update/{" + PROJECT_IDENTIFIER_URLPARAM + "}";
	public static final String UPDATE_DEPLOYMENT_MAY_DEPLOY_URL = "update/{" + PROJECT_IDENTIFIER_URLPARAM + "}/maydeploy";

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#updateDeployment(com.tasktop.c2c.server
	 * .deployment.domain.DeploymentConfiguration)
	 */
	public DeploymentConfiguration updateDeployment(DeploymentConfiguration deploymentConfiguration, boolean mayDeployOnUpdate)
			throws EntityNotFoundException, ValidationException {
		try {
                    if (mayDeployOnUpdate) {
                        return new PostCall<DeploymentConfiguration>() {
                            public DeploymentConfiguration getValue(ServiceCallResult result) {
                                return result.getDeploymentConfiguration();
                            }
                        }.doCall(UPDATE_DEPLOYMENT_MAY_DEPLOY_URL, deploymentConfiguration);
                    } else {
                        return new PostCall<DeploymentConfiguration>() {
                            public DeploymentConfiguration getValue(ServiceCallResult result) {
                                return result.getDeploymentConfiguration();
                            }
                        }.doCall(UPDATE_DEPLOYMENT_URL, deploymentConfiguration);
                    }
		} catch (WrappedCheckedException e) {
			convertValidationException(e);
			throw (e);
		}
	}

	public static final String UNDEPLOY_ALSO_URLPARAM = "undeploy";

	public static final String DELETE_DEPLOYMENT_URL = "delete/{" + PROJECT_IDENTIFIER_URLPARAM + "}/config/{"
			+ CONFIGURATION_IDENTIFIER_URLPARAM + "}/undeploy/{" + UNDEPLOY_ALSO_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#deleteDeployment(java.lang.Long,
	 * boolean)
	 */
	public void deleteDeployment(Long deploymentConfigurationId, boolean alsoDeleteFromCloudFoundry)
			throws EntityNotFoundException, ServiceException {
		try {
			delegate.getForObject(computeUrl(DELETE_DEPLOYMENT_URL), Void.class, deploymentConfigurationId,
					alsoDeleteFromCloudFoundry);
		} catch (WrappedCheckedException e) {
			convertEntityNotFoundException(e);
			throw (e);
		}
	}

	public static final String DEPLOY_DEPLOYMENT_URL = "deploy/{" + PROJECT_IDENTIFIER_URLPARAM + "}/config/{"
			+ CONFIGURATION_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#deploy(java.lang.Long)
	 */
	public DeploymentConfiguration deploy(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException, ValidationException {
		return new GetCall<DeploymentConfiguration>() {
			public DeploymentConfiguration getValue(ServiceCallResult result) {
				return result.getDeploymentConfiguration();
			}
		}.doCall(DEPLOY_DEPLOYMENT_URL);
	}

	public static final String REDEPLOY_DEPLOYMENT_URL = "deploy/{" + PROJECT_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#deploy(java.lang.Long)
	 */
	public DeploymentConfiguration deploy(DeploymentConfiguration deploymentConfiguration) throws EntityNotFoundException,
			ServiceException, ValidationException {
		try {
			return new PostCall<DeploymentConfiguration>() {
				public DeploymentConfiguration getValue(ServiceCallResult result) {
					return result.getDeploymentConfiguration();
				}
			}.doCall(REDEPLOY_DEPLOYMENT_URL, deploymentConfiguration);
		} catch (WrappedCheckedException e) {
			convertValidationException(e);
			throw (e);
		}
	}

	public static final String UNDEPLOY_DEPLOYMENT_URL = "undeploy/{" + PROJECT_IDENTIFIER_URLPARAM + "}/config/{"
			+ CONFIGURATION_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#undeploy(java.lang.Long)
	 */
	public DeploymentConfiguration undeploy(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException, ValidationException {
		return new GetCall<DeploymentConfiguration>() {
			public DeploymentConfiguration getValue(ServiceCallResult result) {
				return result.getDeploymentConfiguration();
			}
		}.doCall(UNDEPLOY_DEPLOYMENT_URL);
	}

	public static final String START_DEPLOYMENT_URL = "start/{" + PROJECT_IDENTIFIER_URLPARAM + "}/config/{"
			+ CONFIGURATION_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#startDeployment(java.lang.Long)
	 */
	public DeploymentConfiguration startDeployment(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException {
		return new GetCall<DeploymentConfiguration>() {
			public DeploymentConfiguration getValue(ServiceCallResult result) {
				return result.getDeploymentConfiguration();
			}
		}.doCall(START_DEPLOYMENT_URL);
	}

	public static final String STOP_DEPLOYMENT_URL = "stop/{" + PROJECT_IDENTIFIER_URLPARAM + "}/config/{"
			+ CONFIGURATION_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#stopDeployment(java.lang.Long)
	 */
	public DeploymentConfiguration stopDeployment(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException {
		return new GetCall<DeploymentConfiguration>() {
			public DeploymentConfiguration getValue(ServiceCallResult result) {
				return result.getDeploymentConfiguration();
			}
		}.doCall(STOP_DEPLOYMENT_URL);
	}

	public static final String RESTART_DEPLOYMENT_URL = "restart/{" + PROJECT_IDENTIFIER_URLPARAM + "}/config/{"
			+ CONFIGURATION_IDENTIFIER_URLPARAM + "}";

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#restartDeployment(java.lang.Long)
	 */
	public DeploymentConfiguration restartDeployment(Long deploymentConfigurationId) throws EntityNotFoundException,
			ServiceException {
		return new GetCall<DeploymentConfiguration>() {
			public DeploymentConfiguration getValue(ServiceCallResult result) {
				return result.getDeploymentConfiguration();
			}
		}.doCall(RESTART_DEPLOYMENT_URL);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#getAvailableServices(com.tasktop.c2c
	 * .server.deployment.domain.DeploymentConfiguration)
	 */
	public List<CloudService> getAvailableServices(DeploymentConfiguration deploymentConfiguration)
			throws ServiceException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#createService(com.tasktop.c2c.server
	 * .deployment.domain.CloudService, com.tasktop.c2c.server.deployment.domain.DeploymentConfiguration)
	 */
	public CloudService createService(CloudService service, DeploymentConfiguration deploymentConfiguration)
			throws ServiceException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#getAvailableMemoryConfigurations(com
	 * .tasktop.c2c.server.deployment.domain.DeploymentConfiguration)
	 */
	public List<Integer> getAvailableMemoryConfigurations(DeploymentConfiguration deploymentCongfiguration)
			throws ServiceException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#getAvailableServiceConfigurations(com
	 * .tasktop.c2c.server.deployment.domain.DeploymentConfiguration)
	 */
	public List<DeploymentServiceConfiguration> getAvailableServiceConfigurations(
			DeploymentConfiguration deploymentConfiguration) throws ServiceException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#onBuildCompleted(java.lang.String,
	 * com.tasktop.c2c.server.profile.domain.build.BuildDetails)
	 */
	public void onBuildCompleted(String jobName, BuildDetails buildDetails) {
		// TODO Auto-generated method stub

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#getSupportedDeploymentServiceTypes()
	 */
	public List<DeploymentServiceType> getSupportedDeploymentServiceTypes() {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * com.tasktop.c2c.server.deployment.service.DeploymentConfigurationService#getRecentActivity(com.tasktop.c2c.server
	 * .common.service.domain.Region)
	 */
	public List<DeploymentConfigActivity> getRecentActivity(Region region) {
		// TODO Auto-generated method stub
		return null;
	}
        
        
	/**
	 * Process DeploymentConfiguration on entry to the API
	 * 
         * @param deploymentConfiguration
	 * @return a potentially modified DeploymentConfiguration
	 */
	public DeploymentConfiguration onEntry(DeploymentConfiguration deploymentConfiguration) {
            return deploymentConfiguration;
        }
        
	/**
	 * Process DeploymentConfiguration on entry to the API
	 * 
         * @param deploymentConfiguration
	 * @return a potentially modified DeploymentConfiguration
	 */
	public DeploymentConfiguration onExit(DeploymentConfiguration deploymentConfiguration) {
            return deploymentConfiguration;            
        }  

}
